
var tabs = require("sdk/tabs");
var notifications = require("sdk/notifications");
var cm = require("sdk/context-menu");
var ss = require('sdk/simple-storage');
var sp = require("sdk/simple-prefs");
var Request = require("sdk/request").Request;

// This makes option page not to show any navigation and toolbar
var addontab = require("sdk/addon-page");
var data = require('sdk/self').data;
var worker;

var myIconUrl = data.url("passlink-32.png");

// context menu
cm.Item({
    label:          "PassLink to mobile device",
    context:        cm.SelectorContext("a[href]"),
    image:			data.url("passlink-16.png"),
    contentScript:  'self.on("click", function(node, data) {'+
                    '  self.postMessage(node.href); ' +
                    '});',
    onMessage:      function(url) {
                        passLink(url, "");
                    },
});


// options page communication 
tabs.on("ready", function(tab) {
	worker = tab.attach({
		contentScriptFile: data.url("options.js")
	});
	
	worker.port.on("pairDevice", function handlePairDevice(pair_code) {
		pairDevice(pair_code);
	});
	
	worker.port.emit("showCurrentDeviceName", ss.storage);
});


// extension options to show options page
sp.on("pairDevice", function() {
  showOptionsPage();
});


// toolbar icon
var tbb = require("toolbarbutton").ToolbarButton({
    id: 'passlink-button',
    label: 'PassLink',
    image: data.url('passlink-16.png'),
	onCommand : function() {
		passLink(tabs.activeTab.url, tabs.activeTab.title);
	},
	onClick : function(e) {
		if (e.button == 1) {
			passLink(tabs.activeTab.url, tabs.activeTab.title);
		}
	}
});

exports.main = function(options) {
    if (options.loadReason === "install") {
        tbb.moveTo({
          toolbarID: "nav-bar",
          insertbefore: "home-button",
          forceMove: false // only move from palette
    });
  }
};


/*
 * Helper functions
 */

function showOptionsPage() {
	tab = tabs.open(data.url("index.html"));
}

function showCurrentDeviceName(storage) {
	worker.port.emit("showCurrentDeviceName", storage);
}

function showError(msg) {
	worker.port.emit("showError", msg);
}

function showNotification(msg) {	
	notifications.notify({
        title: "Passlink",
        text: msg,
        iconUrl: myIconUrl
    });
}


/*
 * API calls
 */

var apiUrl = "http://passlink.singlemaltlabs.com/api/";

//pass url from context menu
function passContext(info, tab) {
  passLink(info.linkUrl, "");
}

// pass url and title
function passLink(url, title) {	
	// verify if we have paired device
	var pair_key = ss.storage.pair_key;
	if (!pair_key) {
		showOptionsPage();
		return;
	}
	
	if (url.indexOf("http:") != 0 && url.indexOf("https:") != 0) {
		showNotification("Cannot pass links other than http(s)!");
		return;
	}
	
	var params = {
		"pair_key" : pair_key,
		"url" : url,
		"title" : title
	};

	params = JSON.stringify(params);
	
	var req = Request({
		url : apiUrl + 'notificationLite',
		content : params,
		contentType : 'application/json',
		contentMimeType : 'text/plain',
		onComplete : function(response) {
			switch (parseInt(response.status)) {
				case 200 : 
					showNotification("Link has been sent!");
					break;
				case 404 :
					showNotification("Cannot find device!");
					break;
				case 400 :
					showNotification("Missing required api parameters!");
					break;
				case 500 :
					showNotification("Internal server error!");
					break;
				default :
					showNotification("Unknown error!");
					break;
			}
		}
	});

	req.post();
}


// pair device depending on pairing code
function pairDevice(pair_code) {
	var req = Request({
		url : apiUrl + 'pairDevice/' + pair_code,
		onComplete : function(response) {
			switch (parseInt(response.status)) {
				case 200:
					ss.storage.pair_key = response.json.pair_key;
					ss.storage.device_name = response.json.name;
					showCurrentDeviceName(ss.storage);
					break;
				case 404:
					showError("Cannot find device for provided pairing code!");
					break;
				case 400:
					showError("Please provide pairing code!");
					break;
				case 500:
					showError("Internal server error!");
					break;
				default:
					showError("Unknown error!");
					break;
			}
		}
	});

	req.get();
}
